@@ -42,6 +42,8 @@ class Conll2003DatasetReader(DatasetReader):
                 url = 'http://files.deeppavlov.ai/deeppavlov_data/vlsp2016.tar.gz'
             elif dataset_name == 'dailydialog':
                 url = 'http://files.deeppavlov.ai/deeppavlov_data/dailydialog.tar.gz'
+            elif dataset_name == 'collection3':
+                url = 'http://files.deeppavlov.ai/deeppavlov_data/collection3_anh.tar.gz'
             else:
                 raise RuntimeError('train.txt not found in "{}"'.format(data_path))
             data_path.mkdir(exist_ok=True, parents=True)
