@@ -18,7 +18,7 @@ import sys
 import tarfile
 from hashlib import md5
 from pathlib import Path
-from typing import List, Dict, Union
+from typing import Dict, Optional, Union
 from zipfile import ZipFile
 
 from deeppavlov.core.data.utils import file_md5
@@ -79,17 +79,10 @@ def compute_hashes(fpath: Union[str, Path]) -> Dict[str, str]:
     return hashes
 
 
-def main(args: List[str] = None) -> None:
-    parser = argparse.ArgumentParser()
-    parser.add_argument("fname", help="path to a file to compute hash for", type=str)
-    parser.add_argument('-o', '--outfile', help='where to write the hashes', default=None, type=str)
-
-    args = parser.parse_args(args)
-
-    p = Path(args.fname).expanduser()
+def main(fname: str, outfile: Optional[str] = None) -> None:
+    p = Path(fname).expanduser()
     hashes = compute_hashes(p)
 
-    outfile = args.outfile
     if outfile is None:
         outfile = p.with_suffix(p.suffix + '.md5').open('w', encoding='utf-8')
     elif outfile == '-':
@@ -105,4 +98,9 @@ def main(args: List[str] = None) -> None:
 
 
 if __name__ == '__main__':
-    main()
+    parser = argparse.ArgumentParser()
+    parser.add_argument("fname", help="path to a file to compute hash for", type=str)
+    parser.add_argument('-o', '--outfile', help='where to write the hashes', default=None, type=str)
+
+    args = parser.parse_args()
+    main(args.fname, args.outfile)
